require 'soar_idm/directory_provider'
require 'soar/registry/staff/directory/error'
require 'aws-sdk'
require 'hashie'
require 'mince'
require 'hashy_db'

module Soar
  module Registry
    module Staff
      module Directory
        class Stub < SoarIdm::DirectoryProvider

          attr_accessor :interface
          attr_writer :network
          attr_writer :connection

          def initialize
            @interface = Mince::HashyDb::Interface
            @network = true
            @connection = true
          end

          ##
          # @param [Hash] configuration for the directory provider
          # @return [Nil]
          # @raise [Soar::Registry::Staff::Directory::Error::BootstrapError] if required configuration is missing
          ##
          def bootstrap(configuration)
            @configuration = configuration
            raise Soar::Registry::Staff::Directory::Error::BootstrapError if configuration.empty?
          end

          ##
          # @return [Bool] whether directory provider received minimum required configuration
          ##
          def bootstrapped?
            true
          end

          def uri
            @configuration['endpoint']
          end

          def authenticate(credentials)
            raise Soar::Registry::Staff::Directory::Error::AuthenticationError, 'missing username' if not credentials.key?('username')
            raise Soar::Registry::Staff::Directory::Error::AuthenticationError, 'missing password' if not credentials.key?('password')
          end

          def connect
            raise Soar::Registry::Staff::Directory::Error::ConnectionError if not @connection
            true
          end

          def connected?
            true
          end

          def ready?
            @network
          end

          def put_identity(entity)
            @interface.add('identities', entity)
          end

          ##
          # @param [String] identity_id primary key of the identity
          # @return [Hash] the identity
          # @raise [Soar::Registry::Staff::Directory::DynamoDb::Error::UniqueIdentifierNotFoundError] if primary key not found
          ##
          def fetch_identity(identity_id)
            identity = @interface.get_for_key_with_value('identities', "identity_id", identity_id)
            raise Soar::Registry::Staff::Directory::Error::UniqueIdentifierNotFoundError if identity.nil?
            identity
          end

          ##
          # @param [String] identifier_attribute 
          # @param [String] identifier_value
          # @return [Array] list of identities
          # @raise [ArgumentError] if query or index is not specified
          ##
          def search_identities(identifier_attribute, identifier_value)
            identities = []
            @interface.find_all('identities').each { |identity| 
              if identity[identifier_attribute] == identifier_value
                identities << identity
              else
                next
              end
            }
            identities
          end

          ##
          # @return [Array] a list of primary keys and global secondary indexes
          ##
          def indexed_attributes
            ["identity_id", "email", "entity_id"]
          end

        end
      end
    end
  end
end
